!+
! Module: efftplot_mod 
!
! Authors:  Didier Barret - Jean-Francois Olive - CESR
!          
! Description: PERFORMS FFT OF EPIC TIME-SERIES
! 
! Comments: 
! 
! $Id: module.f90,v 1.5 1998/01/13 08:39:15 gvacanti Exp $
!- 
module efftplot_mod

  use dal
  use param  
  
  implicit none 
  
  private 

  public efftplot

contains  
  
!---------------------
    subroutine efftplot
!---------------------
     
    integer,parameter :: nb_options = 36
    integer,parameter :: nbusrpara  = 12
    integer,dimension(:),pointer :: l_nb_sub_col
    integer :: nb_pt,nb_col,col,subcol,nbopt,maxopt,Size_Vectors,var1

    character(len=128) :: Input_File,Output_File
    character(len=1000),dimension(4):: Label
    character(len=10 ),dimension(6) :: l_name_col
    character(len=8) :: powspec = 'powspec '
    character(len=128) :: opt1
    character(len=512) :: Commande,opt2
    character(len=15) :: Device_type
    character(len=15),dimension(2,nb_options) :: listoptions
    character(len=1) :: c1,v1

    real(kind=double),dimension(:),pointer :: Freq  
    real(kind=single),dimension(:),pointer :: Pds , Pds_Err 
  
    type( DataSetT ) :: Set

    maxopt = size(listoptions,2)
    label(1) = 'Label 1'
    label(2) = 'Label 2'
    label(3) = 'Label 3'
    label(4) = 'Label 4'

    ! Get the efftplot parameters and the default parameters for powspec
    call get_str_param(Input_File,listoptions,nbopt,maxopt,nbusrpara)


    ! Get information about the input file :
    ! nb_pt : number of bins in the lc
    ! nb_col : number of columns in the RATE extension
    ! l_name_col : name of each column
    ! l_nb_sub_col : number of ``subcolumns'' for each column
    call get_infos(Input_File,Set,nb_pt,nb_col,l_name_col,l_nb_sub_col)  
    

    ! Test of the nbint value (number of bins per interval : a power of 2, less than nb_pt)
    ! Determine nintfm (the number of interval per frame) : the final spectrum must contain only
    ! one averaged spectrum. 

    call calc_param(listoptions,'nbint','nintfm',nb_pt)
    
    Output_File = StringParameter('outfile')
    Device_type = value_option('plotdev',listoptions,var1)

    call open_plot(Device_type, Output_File)
    call create_labels(Set,Label)

    ! For each column of the RATE extension
    do col=1,nb_col
       if (l_name_col(col)(1:4) == 'RATE' ) then
          ! For each ``subcolumns''
          do subcol = 1,l_nb_sub_col(col)
             ! Construction of the powspec command line
             c1 = achar(48+col)
             v1 = achar(48+subcol)
             opt1 = ' vy' // c1 // ' fe' // v1 // ' le' // v1
             call make_cmd_opt(opt2,listoptions,subcol)
             Commande = powspec // '"' // Input_File(1:len_trim(Input_file)) // &
             opt1(1:len_trim(opt1)) // '" ' // opt2(1:len_trim(opt2)) // &
             ' > efftplot.tempo'
             ! run powspec
             call system(Commande)
             ! delete powspec output
             call system('rm efftplot.tempo')
             ! read in the fits file containing the PDS
              call read_pds(Freq,Pds,Pds_Err,Size_Vectors,subcol)
             ! plot it
             call draw_plot(Label, Freq, Pds , Pds_Err, Size_Vectors, subcol,listoptions)
          end do
          ! delete all the PDS fits files
          call system('rm fout_pds*')
       end if
    end do    
    call pgend
  end subroutine efftplot


!----------------------------------------------------------------
  subroutine get_str_param(filein,listopt,nbopt,maxopt,nbusrpara)
!----------------------------------------------------------------

! Makes a 2d string array with containing the options for powspec
! and 2 plot options (x_scale and y_scale)
! The format is : (name of option , value of option).

    character(len=*),intent(out) :: filein
    character(len=*),dimension(2,*),intent(out)  :: listopt
    integer,intent(out) :: nbopt
    integer,intent(in) :: maxopt,nbusrpara
    
    ! The first 11 parameters are efftplot parameters
    ! The remaining 25 are specific powspec parameters with default values

    character(len=15),dimension(11) :: allopttoread = &
         (/ "nbins          ","rebintype      ","polyorder      ", &
            "operation      ","plotdevice     ","outfile        ", &
            "npoints        ", & 
            "nerrorpoints   ","normalization  ", &
            "xscale         ","yscale         " /)    

    character(len=15),dimension(36) :: allopt = &
         (/ "nbint          ","rebin          ","trend_degree   ", &
            "trend_oper     ","device_type    ","outfile        ", &
            "gapfill        ", & 
            "errorbars      ","normalization  ", &
            "x_scale        ","y_scale        ", &    

            "outfiletype    ","plot           ","nintfm         ",&
            "tchat          ","lchat          ","logname        ", &
            "clobber        ","dpath          ","plotfile       ", &
            "forcestart     ","window         ","exposure       ", &
            "outfileroot    ","simultaneous   ","spwinbefore    ", &
            "spwinafter     ","dtnb           ", &
            "rescale        ","offset         ","fast           ", &
            "nser           ","ipow2          ","iavgreb        ", &
            "nbdf           ","mode           " /)
    
    character(len=15),dimension(25) :: valopt = &
         (/ "2              ","no             ","1              ",&
            "10             ","0              ","xronos.log     ", &
            "YES            ","XRDEFAULTS     ","-              ", &
            "no             ","-              ","no             ", &
            "default        ","no             ","0.0            ", &
            "0.0            ","-1             ", &
            "1.0            ","0.0            ","yes            ", &
            "1              ","1              ","0              ", &
            "8192           ","ql             " /)

    character(len=128) :: str 
    character(len=15) :: tmp 
    integer :: indic,i,opt_int,err
    real(kind=single) :: opt_real,opt_real_int

    filein= StringParameter("infile")

    indic = 0
    nbopt = 0 
        
    do indic=1,nbusrpara-1
       tmp = allopttoread(indic)
       select case(indic)
          case(1,3,4,7,8,9)
             opt_int = intParameter(tmp(1:len_trim(tmp))) 
             str = int2asc(opt_int) 
          case(2,5,6,10,11) 
             str =  change_case(stringParameter(tmp(1:len_trim(tmp))),2)
          end select
       nbopt = nbopt+1
       listopt(1,indic) = allopt(indic)
       listopt(2,indic) = str
    end do

    if (.not. booleanParameter("removetrend")) then
       listopt(2,3) = "0"
       listopt(2,4) = "1"
    end if
    if (.not. booleanParameter("rebin")) then
        listopt(2,2) = "0"
    else
        read(listopt(2,2),*,iostat=err) opt_real
        if (err/=0) then
            listopt(2,2) = "0"
            call warning('EFFTPLOT','Invalid rebin option : No rebinning')
        else
            if (opt_real >= 0.0) then
               opt_real_int = anint(opt_real)
               if (opt_real /= opt_real_int) then
                 opt_real = opt_real_int
                 call warning('EFFTPLOT','Invalid positive rebin : Set to nearest integer')
               endif
            else
               if ((opt_real >= -1.0).or.(opt_real < -3.0)) then
                 opt_real = 0.0
                 call warning('EFFTPLOT','Rebin is out of range : No rebinning')
               endif
            end if
            write(listopt(2,2),'(F7.4)') opt_real
        end if
    end if
    if (.not. booleanParameter("fillgaps")) listopt(2,7) = "0"   
    listopt(1,3) = "itre"
    listopt(1,4) = "itremo"
    listopt(1,5) = "plotdev"

    if ((listopt(2,10) /= "LOG") .and. (listopt(2,10) /= "LIN")) then
        call warning('EFFTPLOT','Invalid xscale option : set to LIN')
        listopt(2,10) = "LIN"
    end if 
    if ((listopt(2,11) /= "LOG") .and. (listopt(2,11) /= "LIN")) then
        call warning('EFFTPLOT','Invalid yscale option : set to LIN')
        listopt(2,11) = "LIN"
    end if 

    if (listopt(2,10) == "LOG") then; listopt(2,10)="0";end if 
    if (listopt(2,10) == "LIN") then; listopt(2,10)="1";end if 
    if (listopt(2,11) == "LOG") then; listopt(2,11)="0";end if
    if (listopt(2,11) == "LIN") then; listopt(2,11)="1";end if
    
    i = 0
    do indic=nbusrpara, maxopt
       i = i + 1
       tmp = allopt(indic)
       nbopt = nbopt+1
       listopt(1,indic) = tmp
       listopt(2,indic) = valopt(i)
    end do
                  
  end subroutine get_str_param 

  !-------------------------------------------------------------------------------------
  subroutine create_labels(Ts_Set , Label)
  !-------------------------------------------------------------------------------------
  ! Generates labels to add on the plot.

    type( dataSetT ),intent(in) :: Ts_set
    character(len=1000),dimension(3),intent(out) :: Label

    character(len=12) :: TELESCOP,INSTRUME,DETNAM,DATAMODE,OBSERVER,OBJECT,OBS_ID
    character(len=12) :: DATE_OBS,DATE_END,EXPOSURE,TIMEZERO,TIMEDEL

    type( TableT ) :: Table1 

    Table1 = table(Ts_Set,BlockNumber(Ts_Set,'RATE')) 

    if (HasAttribute(Ts_Set,"TIMEDEL") ) then
         TIMEDEL = StringAttribute(Ts_Set,'TIMEDEL')
    else if (HasAttribute(Table1,"TIMEDEL") ) then
         TIMEDEL = StringAttribute(Table1,'TIMEDEL')
    else
         TIMEDEL = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"TIMEZERO") ) then
         TIMEZERO = StringAttribute(Ts_Set,'TIMEZERO')
    else if (HasAttribute(Table1,"TIMEZERO") ) then
         TIMEZERO = StringAttribute(Table1,'TIMEZERO')
    else
         TIMEZERO = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"EXPOSURE") ) then
         EXPOSURE = StringAttribute(Ts_Set,'EXPOSURE')
    else if (HasAttribute(Table1,"EXPOSURE") ) then
         EXPOSURE = StringAttribute(Table1,'EXPOSURE')
    else
         EXPOSURE = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"DATE-END") ) then
         DATE_END = StringAttribute(Ts_Set,'DATE-END')
    else if (HasAttribute(Table1,"DATE-END") ) then
         DATE_END = StringAttribute(Table1,'DATE-END')
    else
         DATE_END = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"DATE-OBS") ) then
         DATE_OBS = StringAttribute(Ts_Set,'DATE-OBS')
    else if (HasAttribute(Table1,"DATE-OBS") ) then
         DATE_OBS = StringAttribute(Table1,'DATE-OBS')
    else
         DATE_OBS = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"OBS_ID") ) then
         OBS_ID = StringAttribute(Ts_Set,'OBS_ID')
    else if (HasAttribute(Table1,"OBS_ID") ) then
         OBS_ID = StringAttribute(Table1,'OBS_ID')
    else
         OBS_ID = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"OBJECT") ) then
         OBJECT = StringAttribute(Ts_Set,'OBJECT')
    else if (HasAttribute(Table1,"OBJECT") ) then
         OBJECT = StringAttribute(Table1,'OBJECT')
    else
         OBJECT = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"OBSERVER") ) then
         OBSERVER = StringAttribute(Ts_Set,'OBSERVER')
    else if (HasAttribute(Table1,"OBSERVER") ) then
         OBSERVER = StringAttribute(Table1,'OBSERVER')
    else
         OBSERVER = 'Indef'
    endif

   if (HasAttribute(Ts_Set,"DATAMODE") ) then
         DATAMODE = StringAttribute(Ts_Set,'DATAMODE')
    else if (HasAttribute(Table1,"DATAMODE") ) then
         DATAMODE = StringAttribute(Table1,'DATAMODE')
    else
         DATAMODE = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"DETNAM") ) then
         DETNAM = StringAttribute(Ts_Set,'DETNAM')
    else if (HasAttribute(Table1,"DETNAM") ) then
         DETNAM = StringAttribute(Table1,'DETNAM')
    else
         DETNAM = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"INSTRUME") ) then
         INSTRUME = StringAttribute(Ts_Set,'INSTRUME')
    else if (HasAttribute(Table1,"INSTRUME") ) then
         INSTRUME = StringAttribute(Table1,'INSTRUME')
    else
         INSTRUME = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"TELESCOP") ) then
         TELESCOP = StringAttribute(Ts_Set,'TELESCOP')
    else if (HasAttribute(Table1,"TELESCOP") ) then
         TELESCOP = StringAttribute(Table1,'TELESCOP')
    else
         TELESCOP = 'Indef'
    endif

    Label(1) = trim(TELESCOP)//' - '// &
               trim(INSTRUME)//' - '// &
               trim(DETNAM)//' - '// &
               trim(DATAMODE)// &
               ' - PI NAME : '//trim(OBSERVER)// &
               ' - OBJECT : '//trim(OBJECT)// &
               ' - OBS_ID : '//trim(OBS_ID)
    Label(2) = 'START : '//trim(DATE_OBS)//' - '// &
               'STOP : '//trim(DATE_END)
    Label(3) ='SOURCE EXPOSURE TIME='//trim(EXPOSURE)//' secs '&
         //' - TIME ZERO : '//trim(TIMEZERO)//' MJD' &
         //' - BIN TIME : '//trim(TIMEDEL)//' secs'


  end subroutine create_labels

  !-------------------------------------------------------------------------------------
  subroutine make_cmd_opt(opt2,listoptions,pltnb)
  !-------------------------------------------------------------------------------------
  ! construct the option command line
    character(len=512),intent(out) :: opt2
    character(len=15),dimension(:,:),intent(in) :: listoptions
    integer,intent(in) :: pltnb

    character(len=15) :: tmpst1,tmpst2
    integer :: maxopt,i

    maxopt = size(listoptions,2)
    tmpst1 = listoptions(1,1)
    tmpst2 = listoptions(2,1)
    opt2 = tmpst1(1:len_trim(tmpst1)) // '=' // tmpst2(1:len_trim(tmpst2))
    do i=2,maxopt

       tmpst1 = listoptions(1,i)

       if (tmpst1(2:len_trim(tmpst1)) /= '_scale') then 

          tmpst2 = listoptions(2,i)
          if (tmpst1=='outfile       ') tmpst2 = "fout_pds_" // achar(48+pltnb)
          opt2 = opt2(1:len_trim(opt2)) // ' ' // tmpst1(1:len_trim(tmpst1)) // '=' // &
                 tmpst2(1:len_trim(tmpst2))          
       end if
    end do
  end subroutine make_cmd_opt


  !-------------------------------------------------------------------------------------
  subroutine get_infos(fich,Set,nb_pt,nb_col,l_name_col,l_nb_subcol)
  !-------------------------------------------------------------------------------------
  ! Get information in fits table.

    character(len=*),intent(in) :: fich
    character(len=*),dimension(:) :: l_name_col
    integer,dimension(:),pointer :: l_nb_subcol
    integer,intent(out) :: nb_pt,nb_col
    type( DataSetT ),intent(out) :: Set

    type( TableT ) :: Table1 
    type (ColumnT) :: Col_t
    integer :: i
    integer,dimension(:), pointer :: tmp

    Set = DataSet(fich,READ)
    Table1 = table(Set,BlockNumber(Set,'RATE')) 

    ! Check the validy of extension
    if ( .not. HasAttribute(Table1,"HDUCLASS") ) &
      call warning('EFFTPLOT','Keyword HDUCLASS missing in the input file '//fich)
    if ( change_case(StringAttribute(Table1,"HDUCLASS"),2) /= 'OGIP' ) &
      call warning('EFFTPLOT','HDUCLASS is not equal to OGIP')
    if ( .not. HasAttribute(Table1,"HDUCLAS1") ) &
      call warning('EFFTPLOT','Keyword HDUCLAS1 missing in the input file '//fich)
    if ( change_case(StringAttribute(Table1,"HDUCLAS1"),2) /= 'LIGHTCURVE' ) &
      call warning('EFFTPLOT','HDUCLAS1 is not equal to LIGHTCURVE')
    if ( .not. HasAttribute(Table1,"HDUCLAS2") ) &
      call warning('EFFTPLOT','Keyword HDUCLAS2 missing in the input file '//fich)
    if ( change_case(StringAttribute(Table1,"HDUCLAS2"),2) /= 'TOTAL' ) &
      call warning('EFFTPLOT','HDUCLAS2 is not equal to TOTAL')
    if ( .not. HasAttribute(Table1,"HDUCLAS3")) &
      call warning('EFFTPLOT','Keyword HDUCLAS3 missing in the input file '//fich)
    if ( change_case(StringAttribute(Table1,"HDUCLAS3"),2) /= 'RATE' ) &
      call warning('EFFTPLOT','HDUCLAS3 is not equal to RATE')


    nb_col = numberOfColumns(Table1)
    nb_pt = numberOfRows(Table1)
    allocate(l_nb_subcol(nb_col))
    do i=0,nb_col-1 
       Col_t = Column(Table1,i,READ)
       allocate(tmp(size(Dimensions(Col_t))))
       tmp = Dimensions(Col_t)
       l_nb_subcol(i+1) = tmp(1)
       deallocate(tmp)
       l_name_col(i+1) = name(Col_t)
    end do

  end subroutine get_infos


  !----------------------------------------------------------------------------------
  subroutine read_pds(Freq,Pds,Pds_err,Size_Vect,band)
  !----------------------------------------------------------------------------------
  ! Read pds in powspec output fits file.

    
    real(kind=double),dimension(:),pointer ::tmpd
    real(kind=single),dimension(:),pointer ::tmps

    real(kind=double),dimension(:),pointer ::Freq

    real(kind=single),dimension(:),pointer :: Pds , Pds_Err
    integer,intent(out) :: Size_Vect
    integer,intent(in) :: band
    integer :: nfreq

!    real(kind=double),dimension(:),pointer :: tmp

    character(len=1) :: fillin 
    character(len=8) :: fparkey = 'fparkey '
    character(len=9) :: fmodhead = 'fmodhead '
    character(len=10) :: fkeyprint = 'fkeyprint '
    character(len=128) :: fich
    character(len=512) :: command
    type( DataSetT ) :: Set
    type( TableT ) :: Table1 
    type( ColumnT ) ::Col_Freq,Col_Pds,Col_Pds_Err 
    integer :: nheads, length
    real :: rnhead

    fich = "fout_pds_" // achar(48+band)
    fich = fich(1:len_trim(fich)) // ".fps"

!   Change the header information of the files that are written out by powspec,
!   so that they are compatible with the SAS software
   
    Command = fkeyprint // '"' // fich(1:len_trim(fich)) // '[1]' // '"' // ' date ' // &
    ' > newstuff.dat '
    call system(Command)
    Command = fmodhead // '"' // fich(1:len_trim(fich)) // '[0]' // '"' // ' newstuff.dat'
    call system(Command)
    Command = fkeyprint // '"' // fich(1:len_trim(fich)) // '[1]' // '"' // ' creator '// &
    ' > newstuff.dat '
    call system(Command)
    Command = fmodhead // '"' // fich(1:len_trim(fich)) // '[0]' // '"' // ' newstuff.dat'
    call system(Command)
    call system('rm newstuff.dat')

    do nheads=1,2
      rnhead=real(nheads)
      call string_of_real(rnhead, fillin, length)
      Command = fparkey // 'dummy ' // '"' // fich(1:len_trim(fich)) // '[' // &
      fillin // ']' // '"' // ' -date '
      call system(Command)
      Command = fparkey // 'dummy ' // '"' // fich(1:len_trim(fich)) // '[' // & 
      fillin // ']' // '"' // ' -creator '
      call system(Command)
    end do   

    Set = DataSet(fich,READ)
    Table1 = table(Set,"RESULTS") 
    nfreq = numberOfRows(Table1)    

    allocate(freq(0:nfreq-1))
    allocate(Pds(0:nfreq-1))
    allocate(Pds_Err(0:nfreq-1))

    Col_Freq = Column(Table1,"FREQUENCY",Read)
    Col_Pds = Column(Table1, "POWER",Read)
    Col_Pds_Err = Column(Table1,"ERROR",Read)

! THIS THE CORRECT WAY TO READ THE SPECTRUM (FIXES SPR-1482 raised by DB)

    Tmps => real32Data(Col_Pds)
    Pds=Tmps
    Tmps => real32Data(Col_Pds_Err)
    Pds_Err=Tmps
    Tmpd => real64Data(Col_Freq)
    Freq=Tmpd

    Size_vect = size(Pds,1)
    call release(Col_Pds)
    call release(Col_Freq)
    call release (Col_Pds_Err)

  end subroutine read_pds

  !-------------------------------------------------------------------------------------
  subroutine open_plot(Device_Type, Output_File) 
  !-------------------------------------------------------------------------------------
  ! Check Device_Type and Output_File. Open plot if required.

    character(len=*),intent(inout) :: Device_Type , Output_File
    integer :: pgbeg
    character(len=2048) :: Device_Selected
    logical :: File_Exists

    if (Output_File == '')  Output_File='default.ps'
    if (Device_Type(1:3) /= '/XW') then
         inquire(file=Output_File,exist=File_Exists)   
         if (File_Exists .and. .not. clobber()) call error('EFFTPLOT', &
         'Already existing file '//trim(Output_File)//' cannot be overwritten (SAS_CLOBBER=0)')
         if (File_Exists .and. clobber()) call warning('EFFTPLOT', &
         'Already existing file '//trim(Output_File)//' will be overwritten')
         Device_Selected=trim(Output_File)//trim(Device_Type)
         if ( pgbeg(0,Device_Selected,1,1)/=1 ) then
             call warning('EFFTPLOT', &
            'Invalid device and/or output file : '//trim(Device_Type)//' , '//trim(Output_File))
             Output_File='default.ps'
             Device_Type= '/PS'
             Device_Selected=trim(Output_File)//trim(Device_Type)
             if ( pgbeg(0,Device_Selected,1,1)/=1 ) then
                  call error('EFFTPLOT', &
                  'Invalid DEFAULT device and/or output file : '//trim(Device_Type)//' , '&
                  //trim(Output_File))
             else
                  call warning('EFFTPLOT', &
                 'New device and output file : '//trim(Device_Type)//' , '//trim(Output_File))
            endif
         endif
    else
        Device_Selected=trim(Device_Type)
        if ( pgbeg(0,Device_Selected,1,1)/=1 ) call error('EFFTPLOT', &
        'Invalid device : '//trim(Device_Type))
   endif
   call pgpap(6.5 , 1.0 )
   call pgask(.true.)
   call pgsch(0.5)

 end subroutine open_plot

 !-------------------------------------------------------------------------------------
  subroutine draw_plot(Label , Freq0 , Pds0 , Pds_Err0 , Size_Vectors, nrjbd, listoptions)
 !-------------------------------------------------------------------------------------
 ! Draw a plot which contains the power spectrum (Pds0) vs. (Freq0) with error bars (Pds_Err0)

    character(len=1000),dimension(:),intent(in) :: Label
    character(len=15),dimension(:,:),intent(in) :: listoptions
    real(kind=double),dimension(:),pointer :: Freq0
    real(kind=single),dimension(:),pointer :: Pds0 , Pds_Err0
    real(kind=double),dimension(:),pointer :: Freq1
    real(kind=single),dimension(:),pointer :: Pds1 , Pds_Err1
    integer,intent(in) :: Size_Vectors,nrjbd

  ! Local variable Window is used to define the viewports (3 figures/page)
    
    real,dimension(3,4) :: Window
    character(len=15) :: tmp,str
    integer :: Energy_Band , i,xsc,ysc,Svect

    Window(1,:) = (/0.05,0.95,0.65,0.85/)
    Window(2,:) = (/0.05,0.95,0.35,0.55/)
    Window(3,:) = (/0.05,0.95,0.05,0.25/)
    Energy_Band = nrjbd
    if (Energy_Band > 3) then 
      if (Energy_Band == 4)  call pgpage
       Energy_Band = Energy_Band -3
    end if
    call pgsvp(0.0,1.0,0.0,1.0)
    call pgsci(10)
    do i=1,3
       call pgmtxt('T' , -(1. + (i-1)*1.5) , 0.5 , 0.5 , Label(i))
    end do
    call pgsci(11)
    call pgsch(0.75)
    call pgmtxt('T' , -7.*0.5/0.75 , 0.5 , 0.5 , &
         'EPIC SOURCE TIME SERIES FOURIER TRANSFORM')
    call pgsch(0.5)
    tmp = value_option('x_scale',listoptions,i)
    xsc = asc2int(tmp)
    tmp = value_option('y_scale',listoptions,i)
    ysc = asc2int(tmp)

    str = int2asc(nrjbd)
    Svect = Size_Vectors

    ! If the y_scale is log then remove the negative Pds0 values 
    ! and plot nonzero values Pds1 vs. Freq1
    ! else plot Pds0 vs. Pds1

    if (ysc == 0) then 
         call del_bad_value(Freq0,Pds0,Pds_Err0,Svect,Freq1,Pds1,Pds_Err1)
         call make_one_plot(str, Window(Energy_Band,:), Freq1, Pds1 , Pds_Err1, xsc, ysc)
    else
         call make_one_plot(str, Window(Energy_Band,:), Freq0, Pds0 , Pds_Err0, xsc, ysc)
    endif

  end subroutine draw_plot

  !-------------------------------------------------------------------------------------
  subroutine make_one_plot(String , Window , Freq0 , Pds0 , Pds_Err0 ,xsc,ysc)
  !-------------------------------------------------------------------------------------
  ! Draw PDS on Y-axis in function of the frequency (on X-axis).
  ! xsc = 0 ===> X-axis is LOG else X-axis is LIN
  ! ysc = 0 ===> Y-axis is LOG else Y-axis is LIN
  
    character(len=*),intent(in) :: String
    real,dimension(4),intent(in) :: Window
    integer,intent(in) :: xsc,ysc
    real(kind=double),dimension(:),pointer :: Freq0
    real(kind=single),dimension(:),pointer :: Pds0 , Pds_Err0

    real(kind=single),dimension(:),allocatable :: barinf,barsup
    real(kind=single) :: minx,maxx,miny,maxy,fact_y,fact_x,nb_decade
    real(kind=single) :: minimum_value,maximum_value,range

    integer :: i,Size_VEctors

    nb_decade = 0.8
    fact_y = 10 ** nb_decade
    nb_decade = 0.3
    fact_x = 10 ** nb_decade

    Size_VEctors = size(Pds0,1)

    allocate(barinf(Size_VEctors))
    allocate(barsup(Size_VEctors))

    call pgsvp(Window(1),Window(2),Window(3),Window(4))

    if (xsc == 0) then 
       minx = log10(minval(Freq0)/fact_x)
       maxx = log10(maxval(Freq0)*fact_x)
    else 
       minx = 0.0
       maxx = 1.025*(maxval(Freq0))
    endif

    if (ysc == 0) then 

       minimum_value = minval(Pds0)/fact_y
       maximum_value = maxval(Pds0)*fact_y
       miny          = log10(minimum_value)
       maxy          = log10(maximum_value)

       do i=0,size(Pds0,1)-1
          if ( Pds0(i) - Pds_ERR0(i) < minimum_value) then
                barinf(i+1) = minimum_value
          else
                barinf(i+1) = Pds0(i)-Pds_ERR0(i)
          endif
          if ( Pds0(i) + Pds_ERR0(i) > maximum_value) then
                barsup(i+1) = maximum_value
          else
                barsup(i+1) = Pds0(i)+Pds_ERR0(i)
          endif
       end do

    else

       miny  = minval(Pds0-Pds_Err0)
       maxy  = maxval(Pds0+Pds_Err0)
       range = maxy - miny
       miny  = miny - 0.1 * range
       maxy  = maxy + 0.1 * range

       do i=0,size(Pds0,1)-1
           barsup(i+1) = Pds0(i)+Pds_ERR0(i)
           barinf(i+1) = Pds0(i)-Pds_ERR0(i)
       end do

    endif

    call pgswin(minx,maxx,miny,maxy)
    call pgsci(1)
    if (xsc == 0 ) then 
       if (ysc == 0 ) then 
          call pgbox('BCLNST' , 0.0 , 5 ,'BCLNST' , 0.0 , 5)
       else
          call pgbox('BCLNST' , 0.0 , 5 ,'BCNST' , 0.0 , 5)
       endif
    else
       if (ysc == 0 ) then 
          call pgbox('BCNST' , 0.0 , 5 ,'BCLNST' , 0.0 , 5)
       else
          call pgbox('BCNST' , 0.0 , 5 ,'BCNST' , 0.0 , 5)
       endif
    end if
          
    call pglab('Frequency (Hz)' , 'Power' , 'Rate  '//String)
    call pgsci(2)
    if (xsc == 1) then
       if (ysc == 1) then 
          call pgerry(Size_Vectors , real(Freq0) , barinf , barsup,2.0)
       else
          call pgerry(Size_Vectors , real(Freq0) , log10(barinf) , log10(barsup),2.0)
       end if
    else
       if (ysc == 1) then 
          call pgerry(Size_Vectors, log10(real(Freq0)), barinf, barsup,2.0)
       else
          call pgerry(Size_Vectors, log10(real(Freq0)), log10(barinf), log10(barsup),2.0)
            end if
    endif
    call pgsci(1)
    call pgslw(10)
    if (xsc == 1) then
       if (ysc == 1) then 
          call pgpt(Size_Vectors , real(Freq0) , Pds0 , -2)
       else
          call pgpt(Size_Vectors , real(Freq0) , log10(Pds0) , -2)
       end if
    else
       if (ysc == 1) then 
          call pgpt(Size_Vectors , log10(real(Freq0)) , Pds0 , -2)
       else
          call pgpt(Size_Vectors , log10(real(Freq0)) , log10(Pds0) , -2)
       end if
    end if
    call pgslw(1)

    deallocate(barinf)
    deallocate(barsup)

  end subroutine make_one_plot

  !-------------------------------------------------------------------------
  subroutine del_bad_value(Freq0,Pds0,Pds_Err0,Size_VEctors,Freq,Pds,Pds_Err) 
  !-------------------------------------------------------------------------
  ! This subroutine eliminates the negative values of Pds0
  ! On ouput, Pds contains only positive values

    real(kind=single),dimension(:),pointer :: Pds0,Pds_Err0
    real(kind=double),dimension(:),pointer :: Freq0
    integer,intent(inout) :: Size_VEctors

    real(kind=double),dimension(:),pointer :: Freq
    real(kind=single),dimension(:),pointer :: Pds,Pds_Err


    integer :: nb,i,j

    nb=0
    do i=0,size(Freq0,1)-1
       if (Pds0(i)  > 0.) nb =nb +1
    end do

    allocate(Freq(0:nb-1))
    allocate(Pds(0:nb-1))
    allocate(Pds_Err(0:nb-1))

    j=0
    do i=0,size(Freq0,1)-1
       if (Pds0(i)  > 0.) then 
          Freq(j) = Freq0(i)
          Pds(j) = Pds0(i)
          Pds_Err(j) = Pds_Err0(i)
          j = j +1
       end if
    end do
    Size_VEctors = nb

  end subroutine del_bad_value

  !-------------------------------------------------------------------------------------
  subroutine calc_param(listeopt,nbintpara,nintfmpara,Nb_data)
  !-------------------------------------------------------------------------------------
  ! Check and modify if necessary the nbint value
  ! Calculate the value of nintfm 

    character(len=*),dimension(:,:),intent(inout)  :: listeopt
    character(len=*),intent(in) :: nbintpara,nintfmpara
    integer :: Nb_data,nbint_test
     
    character(len=15) :: nbint_str,nintfm_str
    integer :: rank,nbint,nintfm
    
    nbint_str = value_option(nbintpara,listeopt,rank)
    nbint = asc2int(nbint_str)

    if (nbint==1) then 
       ! Full resolution spectrum
       nbint = 2**aint((log(real(Nb_data))/log(2.0)))
       listeopt(2,rank) = int2asc(nbint)
    else
       if (nbint>Nb_data) then 
          ! Full resolution spectrum and warning
          nbint = 2**aint((log(real(Nb_data))/log(2.0)))
          listeopt(2,rank) = int2asc(nbint)
          call warning('EFFTPLOT', & 
                'Not enough bins : '//trim(int2asc(Nb_data))//' data bins.'//&
                ' nbint set to : '// listeopt(2,rank))
       else
          ! Verify if nbint is a power of 2
          nbint_test = 2**anint((log(real(nbint))/log(2.0)))
          if (nbint_test>Nb_data) nbint_test = 2**aint((log(real(nbint))/log(2.0))) 
          if (nbint_test == nbint) then 
              listeopt(2,rank) = int2asc(nbint)
          else
              listeopt(2,rank) = int2asc(nbint_test)
               call warning('EFFTPLOT', & 
                    'nbint is set to the closest power of 2 : '// listeopt(2,rank))
          endif
       end if
    end if
    
    ! Set the value of nintfm

    nintfm_str = value_option(nintfmpara,listeopt,rank)
    nintfm = asc2int(nintfm_str)
! To correct for SPR-1564; the last incomplement segment is removed
!    nintfm = 1 + Nb_data /nbint 
    nintfm = Nb_data /nbint 
    listeopt(2,rank) = int2asc(nintfm)

  end subroutine calc_param

  !-------------------------------------------------------------------------------------
  subroutine string_of_real(Nb , String , Length)
  !-------------------------------------------------------------------------------------
  ! Converts real numbers into plottable character chain. Gives also its length.

    real(kind=single),intent(in) :: Nb
    character(len=1),intent(out) :: String
    integer,intent(out) :: Length
    integer :: MM , puiss

    puiss = max( 0 , min( 6 , 6-int(log10(Nb)) ) )
    MM = nint(Nb*(10**puiss))
    call pgnumb(MM , -puiss , 0 , String , Length)

  end subroutine string_of_real

  !-------------------------------------------------------------------------------------
  function asc2int(str) result(nb)
  !-------------------------------------------------------------------------------------
    character(len=*),intent(in) :: str
    integer :: nb

    integer :: i,lg

    nb = 0
    lg = len_trim(str)
    do i=1,lg
       nb = nb + (10**(lg-i)) * (iachar(str(i:i))-48)
    end do
       
  end function asc2int

  !-------------------------------------------------------------------------------------
  function int2asc(nb0) result(str)
  !-------------------------------------------------------------------------------------
    integer,intent(in) :: nb0
    character(len=15) :: str

    integer :: i,lg,cp,lenstr,posi,deb,nb
    character(len=1) :: tmp
    
    deb = 1
    nb =nb0
    str = "               "
    if (nb == 0) then 
          str(1:1) = "0"
    else
       if (nb < 0) then 
          deb = 2
          str(1:1) = "-"
          nb = -nb
       end if
       lg = aint(log10(real(nb))) + deb
       lenstr = len(str)
       cp = nb
       do i=1,lg 
          tmp = achar(48+mod(cp,10))
          posi = lg-i+deb
          str(posi:posi) = tmp
          cp = cp / 10
       end do     
    end if
  end function int2asc

  !-------------------------------------------------------------------------------------
  function value_option(option,instringtab,rank) result(outstring)
  !-------------------------------------------------------------------------------------
  ! This function returns the value of an 'option' in the table 'instringtab' 

    character(len=*),dimension(:,:),intent(in) :: instringtab
    character(len=*),intent(in) :: option
    character(len=len(option)) :: outstring
    integer,intent(inout) :: rank

    character(len=len(option)) :: tmp
    integer :: i
    
    do i =1, size(instringtab,2) 
       tmp = instringtab(1,i)
       if (tmp(1:len_trim(tmp))==option(1:len_trim(option)))  then 
          outstring = instringtab(2,i)
          rank = i
       end if
    end do
    end function value_option


  !-------------------------------------------------------------------------------------
  function change_case(instring,dir) result(outstring)
  !-------------------------------------------------------------------------------------
  ! Switch from lower to upper case or vice-versa: From Jean Ballet
  ! instring : input string
  ! dir      : direction (1: to lower case; 2: to uppercase; 0: switch case)

    character(len=*), intent(in) :: instring
    integer, intent(in) :: dir
    character(len=len(instring)) :: outstring

  ! ASCII positions of lower case and upper case letters

    integer, parameter :: start_up  = iachar("A"), end_up  = iachar("Z"), &
                          start_low = iachar("a"), end_low = iachar("z"), &
                          offset = start_low - start_up

    integer :: length, icur, pos

    length = len(instring)
    outstring = instring
    do icur = 1, length
       pos = iachar(instring(icur:icur))
       if (pos >= start_up .and. pos <= end_up .and. dir < 2) then
          pos = pos + offset
       else if (pos >= start_low .and. pos <= end_low .and. dir /= 1) then
          pos = pos - offset
       endif
       outstring(icur:icur) = achar(pos)
    enddo

  end function change_case  
  

end module efftplot_mod


